@@ -13,6 +13,11 @@ from sentry_sdk._types import MYPY
 from sentry_sdk.hub import Hub, _should_send_default_pii
 from sentry_sdk.integrations._wsgi_common import _filter_headers
 from sentry_sdk.sessions import auto_session_tracking
+from sentry_sdk.tracing import (
+    SOURCE_FOR_STYLE,
+    TRANSACTION_SOURCE_ROUTE,
+    TRANSACTION_SOURCE_UNKNOWN,
+)
 from sentry_sdk.utils import (
     ContextVar,
     event_from_exception,
@@ -147,6 +152,7 @@ class SentryAsgiMiddleware:
                         transaction = Transaction(op="asgi.server")
 
                     transaction.name = _DEFAULT_TRANSACTION_NAME
+                    transaction.source = TRANSACTION_SOURCE_ROUTE
                     transaction.set_tag("asgi.type", ty)
 
                     with hub.start_transaction(
@@ -183,25 +189,7 @@ class SentryAsgiMiddleware:
         if client and _should_send_default_pii():
             request_info["env"] = {"REMOTE_ADDR": self._get_ip(asgi_scope)}
 
-        if (
-            event.get("transaction", _DEFAULT_TRANSACTION_NAME)
-            == _DEFAULT_TRANSACTION_NAME
-        ):
-            if self.transaction_style == "endpoint":
-                endpoint = asgi_scope.get("endpoint")
-                # Webframeworks like Starlette mutate the ASGI env once routing is
-                # done, which is sometime after the request has started. If we have
-                # an endpoint, overwrite our generic transaction name.
-                if endpoint:
-                    event["transaction"] = transaction_from_function(endpoint)
-            elif self.transaction_style == "url":
-                # FastAPI includes the route object in the scope to let Sentry extract the
-                # path from it for the transaction name
-                route = asgi_scope.get("route")
-                if route:
-                    path = getattr(route, "path", None)
-                    if path is not None:
-                        event["transaction"] = path
+        self._set_transaction_name_and_source(event, self.transaction_style, asgi_scope)
 
         event["request"] = request_info
 
@@ -213,6 +201,44 @@ class SentryAsgiMiddleware:
     # data to your liking it's recommended to use the `before_send` callback
     # for that.
 
+    def _set_transaction_name_and_source(self, event, transaction_style, asgi_scope):
+        # type: (Event, str, Any) -> None
+
+        transaction_name_already_set = (
+            event.get("transaction", _DEFAULT_TRANSACTION_NAME)
+            != _DEFAULT_TRANSACTION_NAME
+        )
+        if transaction_name_already_set:
+            return
+
+        name = ""
+
+        if transaction_style == "endpoint":
+            endpoint = asgi_scope.get("endpoint")
+            # Webframeworks like Starlette mutate the ASGI env once routing is
+            # done, which is sometime after the request has started. If we have
+            # an endpoint, overwrite our generic transaction name.
+            if endpoint:
+                name = transaction_from_function(endpoint) or ""
+
+        elif transaction_style == "url":
+            # FastAPI includes the route object in the scope to let Sentry extract the
+            # path from it for the transaction name
+            route = asgi_scope.get("route")
+            if route:
+                path = getattr(route, "path", None)
+                if path is not None:
+                    name = path
+
+        if not name:
+            # If no transaction name can be found set an unknown source.
+            # This can happen when ASGI frameworks that are not yet supported well are used.
+            event["transaction_info"] = {"source": TRANSACTION_SOURCE_UNKNOWN}
+            return
+
+        event["transaction"] = name
+        event["transaction_info"] = {"source": SOURCE_FOR_STYLE[transaction_style]}
+
     def _get_url(self, scope, default_scheme, host):
         # type: (Dict[str, Any], Literal["ws", "http"], Optional[str]) -> str
         """
